@@ -79,6 +79,9 @@ result file::open(native_string const& f, mode m, creation_flags d)
 		if (d & empty) {
 			dispositionFlags = CREATE_ALWAYS;
 		}
+		else if (d & fresh) {
+			dispositionFlags = CREATE_NEW;
+		}
 		else {
 			dispositionFlags = OPEN_ALWAYS;
 		}
@@ -123,6 +126,8 @@ result file::open(native_string const& f, mode m, creation_flags d)
 			return {result::nospace, err};
 		case ERROR_TOO_MANY_OPEN_FILES:
 			return {result::resource_limit, err};
+		case ERROR_FILE_EXISTS:
+			return {result::preexisting, err};
 		default:
 			return {result::other, err};
 		}
@@ -311,6 +316,9 @@ result file::open(native_string const& f, mode m, creation_flags d)
 		if (d & empty) {
 			flags |= O_TRUNC;
 		}
+		else if (d & fresh) {
+			flags |= O_EXCL;
+		}
 	}
 	int mode = S_IRUSR | S_IWUSR;
 	if (!(d & (current_user_only | current_user_and_admins_only))) {
@@ -332,6 +340,8 @@ result file::open(native_string const& f, mode m, creation_flags d)
 		case EMFILE:
 		case ENFILE:
 			return {result::resource_limit, err};
+		case EEXIST:
+			return {result::preexisting, err};
 		default:
 			return {result::other, err};
 		}
