@@ -180,7 +180,11 @@ local_filesys::type get_file_info_impl(int(*do_stat)(struct stat & buf, char con
 		}
 		else {
 			if (modification_time) {
+#if HAVE_STRUCT_STAT_ST_MTIM
+				*modification_time = datetime(buf.st_mtim.tv_sec, datetime::milliseconds) + fz::duration::from_milliseconds(buf.st_mtim.tv_nsec/1000000);
+#else
 				*modification_time = datetime(buf.st_mtime, datetime::seconds);
+#endif
 			}
 
 			if (mode) {
@@ -197,7 +201,11 @@ local_filesys::type get_file_info_impl(int(*do_stat)(struct stat & buf, char con
 		is_link = false;
 
 	if (modification_time) {
+#if HAVE_STRUCT_STAT_ST_MTIM
+		*modification_time = datetime(buf.st_mtim.tv_sec, datetime::milliseconds) + fz::duration::from_milliseconds(buf.st_mtim.tv_nsec/1000000);
+#else
 		*modification_time = datetime(buf.st_mtime, datetime::seconds);
+#endif
 	}
 
 	if (mode) {
@@ -872,10 +880,11 @@ bool local_filesys::set_modification_time(native_string const& path, datetime co
 	CloseHandle(h);
 	return ret;
 #else
-	utimbuf utm{};
-	utm.actime = t.get_time_t();
-	utm.modtime = utm.actime;
-	return utime(path.c_str(), &utm) == 0;
+	struct timespec times[2]{};
+	times[0].tv_nsec = UTIME_OMIT;
+	times[1].tv_sec = t.get_time_t();
+	times[1].tv_nsec = t.get_milliseconds() * 1000000;
+	return utimensat(AT_FDCWD, path.c_str(), times, 0) == 0;
 #endif
 }
 
@@ -968,6 +977,7 @@ result do_mkdir(native_string const& path, mkdir_permissions permissions)
 	switch (err) {
 	case EACCES:
 	case EPERM:
+	case EROFS:
 		return {result::noperm, err};
 	case EDQUOT:
 	case ENOSPC:
@@ -1066,7 +1076,7 @@ result mkdir(native_string const& absolute_path, bool recurse, mkdir_permissions
 			}
 		}
 		if (!found) {
-			return {result::other};
+			return {result::nodir};
 		}
 
 		// Step 3: Create the segments
@@ -1104,6 +1114,7 @@ result remove_dir(native_string const& absolute_path, bool missing_dir_is_error)
 	if (!RemoveDirectoryW(absolute_path.c_str())) {
 		DWORD const err = GetLastError();
 		switch (err) {
+			case ERROR_FILE_NOT_FOUND:
 			case ERROR_PATH_NOT_FOUND:
 				if (missing_dir_is_error) {
 					return {result::nodir, err};
@@ -1123,6 +1134,7 @@ result remove_dir(native_string const& absolute_path, bool missing_dir_is_error)
 		switch (err) {
 			case EACCES:
 			case EPERM:
+			case EROFS:
 				return {result::noperm, err};
 			case ENOTDIR:
 				return {result::nodir, err};
@@ -1228,6 +1240,7 @@ result rename_file(native_string const& source, native_string const& dest, bool
 	switch (err) {
 	case EPERM:
 	case EACCES:
+	case EROFS:
 		return {result::noperm, err};
 	case ENOSPC:
 	case EDQUOT:
@@ -1261,6 +1274,7 @@ result rename_file(native_string const& source, native_string const& dest, bool
 		switch (err) {
 		case EPERM:
 		case EACCES:
+		case EROFS:
 			return {result::noperm, err};
 		case ENOTDIR:
 			return {result::nodir, err};
